#!BPY

# """
# Name: 'QuickGroup'
# Blender: 240
# Group: 'Mesh'
# Tooltip: 'quick group selection'
# """

"""
Author : Maxime Bourget
Creation :  2006/01/15
Last Update : Wed Jan 18 00:26:40     2006
Version : 1.0


This script allow quick access to vertices group fonction

* <group> button , will select associated vertx
 + Ctrl, add group vertices to current selection
 + Shift , remove group vertices to current selection

* [Set] button will change the group definition with current selection
 + Ctrl,  add current selection to group definition
 + Shift, remove current selection to group definition

* [New] create new group bind to current selection

* AKEY Select/Deselect All
* IKEY  invert
* F5 , R refresh screen
* Q, ESC quite

"""
import Blender
from Blender import Object
from Blender import Draw
from Blender import Window
from Blender import Mesh
from Blender import NMesh

SEP = 2
HB = 18 # button height
HS = HB + SEP

WB = HB*3
WS = WB+SEP
TWB = 100
TWS = WB+SEP

REPLACE=1
ADD=2
SUB=3

SELECT = 0
SHOW = 1
SET = 2
def group_map(f):
	def ret(self): 
		for g in self.groups: f(g)
	return ret

def me(object, event): return 1000*object+event
def de(evt): return evt/1000, evt%1000
O=False

def _refresh_(f):
	""" decorateur gerant les pb de mode d'edition """
	def decorated(*l):
		em = Window.EditMode()
		Window.EditMode(0)
		groupMgr.refreshData()

		f(*l)
		Window.EditMode(em)
		groupMgr.refreshAll()
	return decorated
	
class GroupMgr:
	ctrl=0
	alt=0
	shift=0
	def __init__(self):
		groups = self.findGroups()
		self.groups=[NewGroup('-- NEW ---', 0, self)]
		for i, name in enumerate(groups):
			group = Group(name, i+1, self)
			self.groups.append(group)
	def draw(self):
		#print "drawing", self.groups
		x,y= SEP, SEP
		
		for g in self.groups:
			x, y = g.draw(x, y)
	def event(self, evt, val):
		if evt in [Draw.LEFTCTRLKEY, Draw.RIGHTCTRLKEY]:
			self.ctrl = val
		elif evt in [Draw.LEFTALTKEY, Draw.RIGHTALTKEY]:
			self.alt = val
		elif evt in [Draw.LEFTSHIFTKEY, Draw.RIGHTSHIFTKEY]:
			self.shift=val
		elif evt in [Draw.QKEY, Draw.ESCKEY ]:
			if val: Draw.Exit()
		elif evt in [Draw.F5KEY, Draw.RKEY]:
			self.refreshAll()
		elif evt == Draw.AKEY and val: # all
			self.toggleAll()
		elif evt == Draw.IKEY and val: # invert
			self.invert()
		else: return
		#print "catch=",  evt, val
	
			
	def refreshAll(self):
		self.__init__()
		Draw.Redraw()
		self.ctrl, self.alt, self.shift = 0,0,0
		
	def bevent(self, evt):
		object, event = de(evt)
		self.groups[object].event(event, self.ctrl, self.alt, self.shift)

	def refreshData(self):
		return self.getMesh()
	def getMesh(self):
		try:
			obj = Object.GetSelected()[0]
			self.data = obj.getData()
			return self.data
		except Exception, (ex): print "no data", ex
		
	def getNMesh(self):
		try:

			obj = Object.GetSelected()[0]
			print "NAME=", obj.getName()
			print "NAMES=", NMesh.GetNames()
			
			self.data = NMesh.GetRawFromObject(obj.getName())
			obj.link(self.data)
			print "verts =", len(self.data.verts)
			global O
			O = obj
			return self.data
		except Exception, (ex): print "no data", ex
			
		
	def findGroups(self):
		try:
			self.refreshData()
			return self.data.getVertGroupNames()
		except Exception, (ex):
			print "no groups", ex
		return []
		

	@_refresh_
	def toggleAll(self):
		mode = 1
		for v in self.data.verts:
			if v.sel==0: break
		else: # tous deja selectionne
			mode = 0
		for v in self.data.verts:
			v.sel = mode
		self.data.update()
		
	@_refresh_
	def invert(self):
		for v in self.data.verts:
			v.sel = not v.sel
		self.data.update()
		


class Group:
	def __init__(self, name, i, mgr):
		self.name = name
		self.number = i
		self.mgr = mgr

	def draw(self, x, y):
		x0, y0 = x,y
		ev = me(self.number, SELECT)
		self.title = Draw.PushButton(self.name, ev, x, y, TWB, HB, "Select/Deselect vertices group")
		x+=TWB
		"""
		self.show = Draw.PushButton("Show", ev+SHOW, x, y, WB, HB, "Show/Hide vertices group")
		x+=WB
		"""
		self.set = Draw.PushButton("Set", ev+1, x, y, WB, HB, "Assign/Remove vertices group")
		x+=WB

		return x0, y+HS

	def select(self, mode):
		selected = self.mgr.data.getVertsFromGroup(self.name)
		if not selected: next= -1
		else: next = selected.pop(0)
		for i, v in enumerate(self.mgr.data.verts):
			if i == next:
				if mode == REPLACE: v.sel = 1
				elif mode == ADD: v.sel = 1
				elif mode == SUB: v.sel = 0
				if selected: next = selected.pop(0)
			else:
				if mode == REPLACE: v.sel = 0
				else: v.sel = v.sel
		self.mgr.data.update()

	def assign(self, mode):
		sel = [v.index for v in self.mgr.data.verts if v.sel ]
		if mode == REPLACE: # on efface tout, hack
			self.mgr.data.removeVertGroup(self.name)
			self.mgr.data.addVertGroup(self.name)
			
		mode = {ADD: 'add', SUB:'subtract', REPLACE: 'replace'}[mode]
		
		self.mgr.data.assignVertsToGroup(self.name, sel, 1.0, mode)
	
		
	def select_add(self):
		print "select+", self.name
	def select_remove(self):
		print "select-", self.name
	
	@_refresh_
	def event(self, i, ctrl, alt, shift):
		if i == 0 and not alt:
			if ctrl: self.select(ADD)
			elif shift: self.select(SUB)
			else: self.select(REPLACE)
		elif i == 1 : 
			if ctrl: self.assign(ADD)
			elif shift: self.assign(SUB)
			else: self.assign(REPLACE)

			
class NewGroupPUP(Group):
	@_refresh_
	def event(self, i, ctrl, alt, shift):
		
		group_name = Draw.PupStrInput("Name:", "Group", 40)
		self.mgr.data.addVertGroup(group_name)
		sel = [v.index for v in self.mgr.data.verts if v.sel ]
		print sel
		self.mgr.data.assignVertsToGroup(group_name, sel, 1.0, 'replace')#Mesh.AssignModes.REPLACE)
		self.mgr.data.update()
		self.mgr.refreshAll()

class NewGroup(Group):
	@_refresh_
	def event(self, i, ctrl, alt, shift):
		group_name = self.str.val
		self.mgr.data.addVertGroup(group_name)
		sel = [v.index for v in self.mgr.data.verts if v.sel ]
		self.mgr.data.assignVertsToGroup(group_name, sel, 1.0, 'replace')#Mesh.AssignModes.REPLACE)
		self.mgr.data.update()

	def draw(self, x, y):
		x0, y0 = x,y
		ev = me(self.number, 0)
		self.str = Draw.String("NEW :", ev, x, y, TWB, HB, "group", 40,  "create new group")
		x+=TWB

		return x0, y+HS
		
		

groupMgr = GroupMgr()

Draw.Register(groupMgr.draw, groupMgr.event, groupMgr.bevent)
